#ifndef QTS_HTTP_H
#define QTS_HTTP_H

#include <iostream>
#include <list>
#include <vector>
#include "QtsHttpFileStation.h"
#include "QtsHttpSystem.h"

#ifdef QTSHTTP_EXPORTS
#define QTSHTTPDLL_API __declspec(dllexport)
#define EXPIMP_TEMPLATE
#else
#define QTSHTTPDLL_API __declspec(dllimport)
#define EXPIMP_TEMPLATE extern
#endif

EXPIMP_TEMPLATE template class QTSHTTPDLL_API std::vector<CQtsHttpFileEntry>;
EXPIMP_TEMPLATE template class QTSHTTPDLL_API std::vector<CQtsHttpShareLink>;
EXPIMP_TEMPLATE template class QTSHTTPDLL_API std::vector<CQtsHttpExtractEntry>;

/** CQtsHttpServer support station for Login API parameter. */
typedef enum _QTS_HTTP_STATION_TYPE_
{
	/** System */
	QTS_HTTP_STATION_TYPE_SYSTEM   = 0,	
	/** File station */
	QTS_HTTP_STATION_TYPE_FILE_STATION = 1,		
}QTS_HTTP_STATION_TYPE;

/*!
 * \brief
 * QNAP Turbo NAS server information class.
 * 
 */
class QTSHTTPDLL_API CQtsHttpServerInfo
{
public:
	/** Default constructor */
    CQtsHttpServerInfo();
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] strHostName : Host name.
	 * \param
	 * [IN] strUsername : Username.
	 * \param
	 * [IN] strPassword : Password.
	 * \param
	 * [IN] bSecureConnection : SSL connect.
	 * \param
	 * [IN] strComputerName : Computer name.
	 * 
	 * \returns
	 * n/a.
	 */
    CQtsHttpServerInfo(LPCTSTR strHostName, LPCTSTR strUsername, LPCTSTR strPassword, BOOL bSecureConnection, LPCTSTR strComputerName);

	/** Deconstructor */
	~CQtsHttpServerInfo();

	/*!
	 * \brief
	 * Get host name.
	 * 
	 * \returns
	 * LPCTSTR : Host name.
	 */
    LPCTSTR GetHostName();
	/*!
	 * \brief
	 * Get user name.
	 * 
	 * \returns
	 * LPCTSTR : Username.
	 */
    LPCTSTR GetUsername();
	/*!
	 * \brief
	 * Get password.
	 * 
	 * \returns
	 * LPCTSTR : Password.
	 */
    LPCTSTR GetPassword();
	/*!
	 * \brief
	 * Is SSL connect.
	 * 
	 * \returns
	 * TRUE : SSL connect.
	 * FALSE : non-SSL connect.
	 */
    BOOL IsSecureMode();
	/*!
	 * \brief
	 * Get computer name.
	 * 
	 * \returns
	 * LPCTSTR : Computer name.
	 */
    LPCTSTR GetComputerName();

private:
	BOOL			m_bSecureConnection;		/** SSL connect */
    std::wstring 	m_strHostName;				/** Host name */
    std::wstring	m_strUsername;				/** Username */
    std::wstring 	m_strPassword;				/** Password */
    std::wstring 	m_strComputerName;			/** Computer name */
};

/*!
 * \brief
 * QTS http server class.
 * 
 * 1. Support to login QNAP Turbo NAS system and file station.
 * 2. Open and close QNAP Turbo NAS system/file station.
 */
class QTSHTTPDLL_API CQtsHttpServer
{
public:
	/** Default constructor */
	CQtsHttpServer();
	/*!
	 * \brief
	 * Constructor with CQtsHttpServerInfo server info.
	 * 
	 * \param 
	 * [IN] ServerInfo : Qts http server info.
	 * 
	 * \returns
	 * n/a.
	 */
	CQtsHttpServer(CQtsHttpServerInfo ServerInfo);
	/** Deconstructor */
	~CQtsHttpServer();

	/*!
	* \brief
	* Developers can change HTTP agent name before call QTS NAS system/file station API.
	* 
	* \param
	* [IN] strAgentName : New HTTP agent name.
	* 
	* \returns
	* TRUE : set HTTP agent name success.
	* \returns
	* FALSE : set HTTP agent name fail.
	* 
	* \remarks
	* If no change HTTP agent name, 
	* \remarks
	* QTS HTTP API will use default agent name to sent QTS NAS system/file station request.
	*/
	BOOL	SetAgentName(LPCTSTR strAgentName);

	/*!
	* \brief
	* Developers can change HTTP time-out value before call QTS NAS system/file station API.
	* 
	* \param
	* [IN] nTimeOutMilliseconds : HTTP time-out value, in milliseconds.
	* 
	* \returns
	* TRUE : set HTTP time-out value success.
	* \returns
	* FALSE : set HTTP time-out value fail.
	* 
	* \remarks
	* ConnectTimeout : nTimeOutMilliseconds
	* \remarks
	* SendTimeout : nTimeOutMilliseconds
	* \remarks
	* ReceiveTimeout : nTimeOutMilliseconds
	* \remarks
	* If no change HTTP time-out value, 
	* \remarks
	* QTS HTTP API will use default time-out value (30000).
	*/
	BOOL	SetTimeouts(int nTimeOutMilliseconds);

	/*!
	 * \brief
	 * Login NAS system/file station of QNAP Turbo NAS Server.
	 * 
	 * \param 
	 * [IN] loginStation : Login station type.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	QTS_HTTP_API_RESULT	Login(QTS_HTTP_STATION_TYPE loginStation);

	//File station part

	/*!
	 * \brief
	 * Set file station connection port number.
	 * 
	 * \param 
	 * [IN] lPortNumber : File station port number.
	 * 
	 * \returns
	 * TRUE : Set success.
	 * \returns
	 * FALSE : Set fail.
	 *
	 * \remarks
	 * If no set port number, 
	 * \remarks
	 * QTS file station API will use QNAP default port number to connect.
	 * \remarks
	 * Non-SSL : 8080
	 */
	BOOL SetFileStationPortNum(long lPortNumber);

	/*!
	 * \brief
	 * Set file station SSL connection port number.
	 * 
	 * \param 
	 * [IN] lSSLPortNumber : File station SSL port number.
	 * 
	 * \returns
	 * TRUE : Set success.
	 * \returns
	 * FALSE : Set fail.
	 *
	 * \remarks
	 * If no set SSL port number, 
	 * \remarks
	 * QTS file station API will use QNAP default SSL port number to connect.
	 * \remarks
	 * SSL : 443
	 */
	BOOL SetFileStationSSLPortNum(long lSSLPortNumber);

	/*!
	 * \brief
	 * Query file station SDK version.
	 * 
	 * \param 
	 * n/a
	 * 
	 * \returns
	 * QTS_HTTP_FILE_STATION_API_VERSION : File station API version.
	 */
	QTS_HTTP_FILE_STATION_API_VERSION	QueryFileStationVersion();

	/*!
	 * \brief
	 * OpenFileStation if QTS_FILESTATION_VERSION = QTS_FILESTATION to get file station object.
	 * 
	 * \param 
	 * n/a
	 * 
	 * \returns
	 * IQtsHttpFileStation* : File station object pointer.
	 */
	IQtsHttpFileStation*	OpenFileStation();

	/*!
	 * \brief
	 * CloseFileStation if QTS_FILESTATION_VERSION = QTS_FILESTATION to close and destroy file station object.
	 * 
	 * \param 
	 * [IN] pFileStation : File station object.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code..
	 */
	QTS_HTTP_API_RESULT 	CloseFileStation(IQtsHttpFileStation** pFileStation);

	//System part

	/*!
	 * \brief
	 * Set NAS system connection port number.
	 * 
	 * \param 
	 * [IN] lPortNumber : NAS system port number.
	 * 
	 * \returns
	 * TRUE : Set success.
	 * \returns
	 * FALSE : Set fail.
	 *
	 * \remarks
	 * If no set port number, 
	 * \remarks
	 * QTS NAS system API will use QNAP default port number to connect.
	 * \remarks
	 * Non-SSL : 8080
	 */
	BOOL SetSystemPortNum(long lPortNumber);

	 /*!
	 * \brief
	 * Set NAS system SSL connection port number.
	 * 
	 * \param 
	 * [IN] lSSLPortNumber : NAS system SSL port number.
	 * 
	 * \returns
	 * TRUE : Set success.
	 * \returns
	 * FALSE : Set fail.
	 *
	 * \remarks
	 * If no set SSL port number, 
	 * \remarks
	 * QTS NAS system API will use QNAP default SSL port number to connect.
	 * \remarks
	 * SSL : 443
	 */
	BOOL SetSystemSSLPortNum(long lSSLPortNumber);

	/*!
	 * \brief
	 * Query NAS system SDK version.
	 * 
	 * \param 
	 * n/a
	 * 
	 * \returns
	 * QTS_HTTP_SYSTEM_API_VERSION : NAS system API version.
	 */
	QTS_HTTP_SYSTEM_API_VERSION	QuerySystemVersion();

	/*!
	 * \brief
	 * OpenSystem if QTS_SYSTEM_VERSION = QTS_SYSTEM to get NAS system object.
	 * 
	 * \param 
	 * n/a
	 * 
	 * \returns
	 * IQtsHttpSystem* : NAS system object pointer.
	 */
	IQtsHttpSystem*	OpenSystem();

	/*!
	 * \brief
	 * CloseSystem if QTS_SYSTEM_VERSION = QTS_SYSTEM to close and destroy NAS system object.
	 * 
	 * \param 
	 * [IN] pSystem : NAS system object.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code..
	 */
	QTS_HTTP_API_RESULT CloseSystem(IQtsHttpSystem** pSystem);

private:
	CQtsHttpServerInfo m_ServerInfo;
	IQtsHttpFileStation*	m_pFSObj;
	IQtsHttpSystem*	m_pSYSObj;
	long		m_lFSPortNum;
	long		m_lFSSSLPortNum;
	long		m_lSystemPortNum;
	long		m_lSystemSSLPortNum;
	std::wstring m_strAgentName;
	int			m_nTimeOutMilliseconds;
	QTS_HTTP_FILE_STATION_API_VERSION	m_eFSVersion;
	QTS_HTTP_SYSTEM_API_VERSION	m_eSYSVersion;
};

#endif	// QTS_HTTP_H