#ifndef QTS_HTTP_SYSTEM_H
#define QTS_HTTP_SYSTEM_H

#include "QtsHttpComm.h"

#ifdef QTSHTTP_EXPORTS
#define QTSHTTPDLL_API __declspec(dllexport)
#define EXPIMP_TEMPLATE
#else
#define QTSHTTPDLL_API __declspec(dllimport)
#define EXPIMP_TEMPLATE extern
#endif

/** QTS HTTP system map to SDK version. */
typedef enum _QTS_HTTP_SYSTEM_API_VERSION_
{
	/** Unknow version */
	QTS_HTTP_SYSTEM_API_VERSION_UNKNOW = 0,	
	/** QTS HTTP system API Version 1 */
	QTS_HTTP_SYSTEM_API_V1   = 1,					
	//QTS_HTTP_SYSTEM_API_V2 = 2,
}QTS_HTTP_SYSTEM_API_VERSION;

/** System API type for async mode response context */
typedef enum _QTS_HTTP_SYSTEM_API_TYPE_
{
	/** Unknow API */
	QTS_HTTP_SYSTEM_API_TYPE_UNKNOW = 0,	
	/** GetSystemSettingAsync API */
	QTS_HTTP_SYSTEM_API_TYPE_GETSYSTEMSETTING,		
	/** EnableFileStationAsync API */
	QTS_HTTP_SYSTEM_API_TYPE_ENABLE_FILESTATION,	
}QTS_HTTP_SYSTEM_API_TYPE;

/*!
 * \brief
 * Context for QTS HTTP system response callback.
 * 
 */
typedef struct QTS_HTTP_SYSTEM_ASYNC_RESPONSE_CTX_
{
	/** QTS HTTP system API result */
	QTS_HTTP_API_RESULT		 lResult;	
	/** QTS HTTP system API type */
	QTS_HTTP_SYSTEM_API_TYPE eAPIType;	
	/** QTS HTTP system request */
	LPVOID					 lpSYSRequest;
	/** QTS HTTP system callback function owner */
	LPVOID					 lpData;				
} QTS_HTTP_SYSTEM_ASYNC_RESPONSE_CTX, *PQTS_HTTP_SYSTEM_ASYNC_RESPONSE_CTX;

/*!
 * \brief
 * QTS HTTP system callback function for async mode API.
 * 
 * \param
 * [OUT] PQTS_HTTP_SYSTEM_ASYNC_RESPONSE_CTX : Context for QTS HTTP system response
 * 
 * \returns
 * n/a
 */
typedef void (__stdcall *PFQTS_HTTP_SYSTEM_ASYNC_RESPONSE_CALLBACK)(PQTS_HTTP_SYSTEM_ASYNC_RESPONSE_CTX);

/*!
 * \brief
 * QTS system setting information class
 * 
 */
class QTSHTTPDLL_API CQtsHttpSystemSettingInfo
{
public:
	/** Default constructor */
	CQtsHttpSystemSettingInfo();
	/*!
	 * \brief
	 * Constructor with parameters.
	 * 
	 * \param 
	 * [IN] strServerName : QNAP Turbo NAS server name.
	 * \param 
	 * [IN] nPort : System port number.
	 * \param 
	 * [IN] bSSL : Enable SSL or not.
	 * \param 
	 * [IN] nSSLPort : SSL port number.
	 * \param 
	 * [IN] bSSLForce : Force use SSL or not.
	 * 
	 * \returns
	 * n/a.
	 */
	CQtsHttpSystemSettingInfo(LPCTSTR strServerName, int nPort, BOOL bSSL, int nSSLPort, BOOL bSSLForce);

	/*!
	 * \brief
	 * Get QNAP Turbo NAS server name.
	 * 
	 * \returns
	 * LPCTSTR : QNAP Turbo NAS server name.
	 */
	LPCTSTR GetServerName();
	/*!
	 * \brief
	 * Get system port number.
	 * 
	 * \returns
	 * int : System port number.
	 */
	int		GetSYSPort();
	/*!
	 * \brief
	 * Is enable SSL or not.
	 * 
	 * \returns
	 * TRUE : Enable SSL.
	 * \returns
	 * FALSE : Disable SSL.
	 */
	BOOL	IsSSL();
	/*!
	 * \brief
	 * Get system SSL port number.
	 * 
	 * \returns
	 * int : system SSL port number.
	 */
	int		GetSYSSSLPort();
	/*!
	 * \brief
	 * Force use SSL or not.
	 * 
	 * \returns
	 * TRUE : Force use SSL.
	 * \returns
	 * FALSE : Force use non-SSL.
	 */
	BOOL	IsSSLForce();

	/*!
	 * \brief
	 * Set QNAP Turbo NAS server name.
	 * 
	 * \param
	 * [IN] strServerName : QNAP Turbo NAS server name.
	 * 
	 * \returns
	 * n/a.
	 */
	void	SetServerName(LPCTSTR strServerName);
	/*!
	 * \brief
	 * Set system port number.
	 * 
	 * \param
	 * [IN] nPort : System port number.
	 * 
	 * \returns
	 * n/a.
	 */
	void	SetSYSPort(int nPort);
	/*!
	 * \brief
	 * Set enable SSL.
	 * 
	 * \param
	 * [IN] bSSL : Enable SSL or not.
	 * 
	 * \returns
	 * n/a.
	 */
	void	SetSSL(BOOL bSSL);
	/*!
	 * \brief
	 * Set system SSL port number.
	 * 
	 * \param
	 * [IN] nSSLPort : SSL port number.
	 * 
	 * \returns
	 * n/a.
	 */
	void	SetSYSSSLPort(int nSSLPort);
	/*!
	 * \brief
	 * Set force use SSL or not.
	 * 
	 * \param
	 * [IN] bSSLForce : Force use SSL or not.
	 * 
	 * \returns
	 * n/a.
	 */
	void	SetSSLForce(BOOL bSSLForce);

private:
	std::wstring		m_strServerName;		/** QNAP Turbo NAS server name */
	int			m_nSYSPort;				/** System port number */
	BOOL		m_bSSL;					/** Enable SSL or not */
	int			m_nSSLPort;				/** SSL port number */
	BOOL		m_bSYSSSLForce;			/** Force use SSL or not */
};

/*!
 * \brief
 * Interface for system request base
 *  
 * 1. One system API map to one system request.
 * 2. Abort system request.
 * 3. Get response after system API success.
 */
class IQtsHttpSystemRequestBase
{
public:
	/*!
	 * \brief
	 * Developer can abort system API by calling Abort() anytime.
	 * 
	 * \returns
	 * TRUE : Abort success.
	 * \returns
	 * FALSE : Abort fail.
	 */
	virtual BOOL Abort() = 0;

	/*!
	 * \brief
	 * Get system setting information after GetSystemSettingAsync API success.
	 * 
	 * \returns
	 * CQtsHttpSystemSettingInfo : Output system setting of QNAP Turbo NAS.
	 */
	virtual CQtsHttpSystemSettingInfo ResponseSystemSetting() = 0;
};

/*!
 * \brief
 * Interface for Qts Http system API
 *  
 * 1. Support system related API.
 * 2. Support sync and async API.
 */
class IQtsHttpSystem
{
public:
	/*!
	 * \brief
	 * Get system setting of QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * \param
	 * [OUT] SystemSettingInfo : System setting information of QNAP Turbo NAS.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  GetSystemSetting(BOOL* pbCancel, CQtsHttpSystemSettingInfo &SystemSettingInfo) = 0;

	/*!
	 * \brief
	 * Enable file station service of QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] pbCancel : Cancel API.
	 * 
	 * \returns
	 * QTS_HTTP_API_RESULT : API success or fail error code.
	 */
	virtual	QTS_HTTP_API_RESULT  EnableFileStation(BOOL* pbCancel) = 0;

	/*!
	* \brief
	* Destroy system request after system API complete.
	* 
	* \param 
	* [IN] request : System request.
	* 
	* \returns
	* TRUE : Destroy system request success.
	* \returns
	* FALSE : Destroy system request fail.
	*/
	virtual	BOOL DestroyRequest(IQtsHttpSystemRequestBase** request) = 0;

	/*!
	 * \brief
	 * [Async] Get system setting of QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpSYSCallback : System async callback function.
	 * 
	 * \returns
	 * IQtsHttpSystemRequestBase* : System request pointer object.
	 */
	virtual	IQtsHttpSystemRequestBase*  GetSystemSettingAsync(LPVOID lpData = NULL, PFQTS_HTTP_SYSTEM_ASYNC_RESPONSE_CALLBACK pfHttpSYSCallback = NULL) = 0;

	/*!
	 * \brief
	 * [Async] Enable file station service of QNAP Turbo NAS.
	 * 
	 * \param 
	 * [IN] lpData : Callback function owner.
	 * \param
	 * [IN] pfHttpSYSCallback : System async callback function.
	 * 
	 * \returns
	 * IQtsHttpSystemRequestBase* : System request pointer object.
	 */
	virtual	IQtsHttpSystemRequestBase*  EnableFileStationAsync(LPVOID lpData = NULL, PFQTS_HTTP_SYSTEM_ASYNC_RESPONSE_CALLBACK pfHttpSYSCallback = NULL) = 0;

};

#endif	// QTS_HTTP_SYSTEM_H